@@ -305,7 +305,11 @@ static QByteArray make_input(const QStringList &files, char sep)
 {
     QByteArray result;
     for (const QString &file : files) {
+#ifdef Q_OS_WIN
+        result += file.toUtf8();
+#else
         result += QFile::encodeName(file);
+#endif
         result += sep;
     }
     return result;
@@ -403,7 +407,7 @@ std::vector<std::shared_ptr<ChecksumDefinition>> ChecksumDefinition::getChecksum
 std::shared_ptr<ChecksumDefinition>
 ChecksumDefinition::getDefaultChecksumDefinition(const std::vector<std::shared_ptr<ChecksumDefinition>> &checksumDefinitions)
 {
-    const KConfigGroup group(KSharedConfig::openConfig(), "ChecksumOperations");
+    const KConfigGroup group(KSharedConfig::openConfig(), QStringLiteral("ChecksumOperations"));
     const QString checksumDefinitionId = group.readEntry(CHECKSUM_DEFINITION_ID_ENTRY, QStringLiteral("sha256sum"));
 
     if (!checksumDefinitionId.isEmpty()) {
@@ -426,7 +430,7 @@ void ChecksumDefinition::setDefaultChecksumDefinition(const std::shared_ptr<Chec
     if (!checksumDefinition) {
         return;
     }
-    KConfigGroup group(KSharedConfig::openConfig(), "ChecksumOperations");
+    KConfigGroup group(KSharedConfig::openConfig(), QStringLiteral("ChecksumOperations"));
     group.writeEntry(CHECKSUM_DEFINITION_ID_ENTRY, checksumDefinition->id());
     group.sync();
 }
