@@ -13,6 +13,8 @@
     SPDX-License-Identifier: GPL-2.0-or-later
 */
 
+#include <config-libkleo.h>
+
 #include "dn.h"
 
 #include "oidmap.h"
@@ -26,7 +28,8 @@
 #define strcasecmp _stricmp
 #endif
 
-namespace {
+namespace
+{
 static const QStringList defaultOrder = {
     QStringLiteral("CN"),
     QStringLiteral("L"),
@@ -68,13 +71,15 @@ private:
 class Kleo::DN::Private
 {
 public:
-    Private() : mRefCount(0) {}
+    Private()
+        : mRefCount(0)
+    {
+    }
     Private(const Private &other)
-        : attributes(other.attributes),
-          reorderedAttributes(other.reorderedAttributes),
-          mRefCount(0)
+        : attributes(other.attributes)
+        , reorderedAttributes(other.reorderedAttributes)
+        , mRefCount(0)
     {
-
     }
 
     int ref()
@@ -99,6 +104,7 @@ public:
 
     DN::Attribute::List attributes;
     DN::Attribute::List reorderedAttributes;
+
 private:
     int mRefCount;
 };
@@ -113,16 +119,12 @@ struct DnPair {
 
 // copied from CryptPlug and adapted to work on DN::Attribute::List:
 
-#define digitp(p)   (*(p) >= '0' && *(p) <= '9')
-#define hexdigitp(a) (digitp (a)                     \
-                      || (*(a) >= 'A' && *(a) <= 'F')  \
-                      || (*(a) >= 'a' && *(a) <= 'f'))
-#define xtoi_1(p)   (*(p) <= '9'? (*(p)- '0'): \
-                     *(p) <= 'F'? (*(p)-'A'+10):(*(p)-'a'+10))
-#define xtoi_2(p)   ((xtoi_1(p) * 16) + xtoi_1((p)+1))
+#define digitp(p) (*(p) >= '0' && *(p) <= '9')
+#define hexdigitp(a) (digitp(a) || (*(a) >= 'A' && *(a) <= 'F') || (*(a) >= 'a' && *(a) <= 'f'))
+#define xtoi_1(p) (*(p) <= '9' ? (*(p) - '0') : *(p) <= 'F' ? (*(p) - 'A' + 10) : (*(p) - 'a' + 10))
+#define xtoi_2(p) ((xtoi_1(p) * 16) + xtoi_1((p) + 1))
 
-static char *
-trim_trailing_spaces(char *string)
+static char *trim_trailing_spaces(char *string)
 {
     char *p;
     char *mark;
@@ -146,8 +148,7 @@ trim_trailing_spaces(char *string)
 /* Parse a DN and return an array-ized one.  This is not a validating
    parser and it does not support any old-stylish syntax; gpgme is
    expected to return only rfc2253 compatible strings. */
-static const unsigned char *
-parse_dn_part(DnPair *array, const unsigned char *string)
+static const unsigned char *parse_dn_part(DnPair *array, const unsigned char *string)
 {
     const unsigned char *s;
     const unsigned char *s1;
@@ -159,11 +160,11 @@ parse_dn_part(DnPair *array, const unsigned char *string)
         ;
     }
     if (!*s) {
-        return nullptr;    /* error */
+        return nullptr; /* error */
     }
     n = s - string;
     if (!n) {
-        return nullptr;    /* empty key */
+        return nullptr; /* empty key */
     }
     p = (char *)malloc(n + 1);
 
@@ -171,12 +172,9 @@ parse_dn_part(DnPair *array, const unsigned char *string)
     p[n] = 0;
     trim_trailing_spaces((char *)p);
     // map OIDs to their names:
-    for (unsigned int i = 0; i < numOidMaps; ++i) {
-        if (!strcasecmp((char *)p, oidmap[i].oid)) {
-            free(p);
-            p = strdup(oidmap[i].name);
-            break;
-        }
+    if (const char *name = Kleo::attributeNameForOID(p)) {
+        free(p);
+        p = strdup(name);
     }
     array->key = p;
     string = s + 1;
@@ -184,12 +182,11 @@ parse_dn_part(DnPair *array, const unsigned char *string)
     if (*string == '#') {
         /* hexstring */
         string++;
-        for (s = string; hexdigitp(s); s++) {
-            s++;
-        }
+        for (s = string; hexdigitp(s); s++)
+            ;
         n = s - string;
         if (!n || (n & 1)) {
-            return nullptr;    /* empty or odd number of digits */
+            return nullptr; /* empty or odd number of digits */
         }
         n /= 2;
         array->value = p = (char *)malloc(n + 1);
@@ -204,20 +201,17 @@ parse_dn_part(DnPair *array, const unsigned char *string)
             if (*s == '\\') {
                 /* pair */
                 s++;
-                if (*s == ',' || *s == '=' || *s == '+'
-                        || *s == '<' || *s == '>' || *s == '#' || *s == ';'
-                        || *s == '\\' || *s == '\"' || *s == ' ') {
+                if (*s == ',' || *s == '=' || *s == '+' || *s == '<' || *s == '>' || *s == '#' || *s == ';' || *s == '\\' || *s == '\"' || *s == ' ') {
                     n++;
                 } else if (hexdigitp(s) && hexdigitp(s + 1)) {
                     s++;
                     n++;
                 } else {
-                    return nullptr;    /* invalid escape sequence */
+                    return nullptr; /* invalid escape sequence */
                 }
             } else if (*s == '\"') {
-                return nullptr;    /* invalid encoding */
-            } else if (*s == ',' || *s == '=' || *s == '+'
-                       || *s == '<' || *s == '>' || *s == '#' || *s == ';') {
+                return nullptr; /* invalid encoding */
+            } else if (*s == ',' || *s == '=' || *s == '+' || *s == '<' || *s == '>' || *s == '#' || *s == ';') {
                 break;
             } else {
                 n++;
@@ -247,30 +241,28 @@ parse_dn_part(DnPair *array, const unsigned char *string)
 /* Parse a DN and return an array-ized one.  This is not a validating
    parser and it does not support any old-stylish syntax; gpgme is
    expected to return only rfc2253 compatible strings. */
-static Kleo::DN::Attribute::List
-parse_dn(const unsigned char *string)
+static Kleo::DN::Attribute::List parse_dn(const unsigned char *string)
 {
     if (!string) {
-        return QVector<Kleo::DN::Attribute>();
+        return QList<Kleo::DN::Attribute>();
     }
 
-    QVector<Kleo::DN::Attribute> result;
+    QList<Kleo::DN::Attribute> result;
     while (*string) {
         while (*string == ' ') {
             string++;
         }
         if (!*string) {
-            break;    /* ready */
+            break; /* ready */
         }
 
-        DnPair pair = { nullptr, nullptr };
+        DnPair pair = {nullptr, nullptr};
         string = parse_dn_part(&pair, string);
         if (!string) {
             goto failure;
         }
         if (pair.key && pair.value) {
-            result.push_back(Kleo::DN::Attribute(QString::fromUtf8(pair.key),
-                                                 QString::fromUtf8(pair.value)));
+            result.push_back(Kleo::DN::Attribute(QString::fromUtf8(pair.key), QString::fromUtf8(pair.value)));
         }
         free(pair.key);
         free(pair.value);
@@ -279,7 +271,7 @@ parse_dn(const unsigned char *string)
             string++;
         }
         if (*string && *string != ',' && *string != ';' && *string != '+') {
-            goto failure;    /* invalid delimiter */
+            goto failure; /* invalid delimiter */
         }
         if (*string) {
             string++;
@@ -288,11 +280,10 @@ parse_dn(const unsigned char *string)
     return result;
 
 failure:
-    return QVector<Kleo::DN::Attribute>();
+    return QList<Kleo::DN::Attribute>();
 }
 
-static QVector<Kleo::DN::Attribute>
-parse_dn(const QString &dn)
+static QList<Kleo::DN::Attribute> parse_dn(const QString &dn)
 {
     return parse_dn((const unsigned char *)dn.toUtf8().data());
 }
@@ -312,7 +303,7 @@ static QString dn_escape(const QString &s)
         case ';':
             result += QLatin1Char('\\');
             // fall through
-            Q_FALLTHROUGH();
+            [[fallthrough]];
         default:
             result += ch;
         }
@@ -320,11 +311,10 @@ static QString dn_escape(const QString &s)
     return result;
 }
 
-static QString
-serialise(const QVector<Kleo::DN::Attribute> &dn, const QString &sep)
+static QString serialise(const QList<Kleo::DN::Attribute> &dn, const QString &sep)
 {
     QStringList result;
-    for (QVector<Kleo::DN::Attribute>::const_iterator it = dn.begin(); it != dn.end(); ++it) {
+    for (QList<Kleo::DN::Attribute>::const_iterator it = dn.begin(); it != dn.end(); ++it) {
         if (!(*it).name().isEmpty() && !(*it).value().isEmpty()) {
             result.push_back((*it).name().trimmed() + QLatin1Char('=') + dn_escape((*it).value().trimmed()));
         }
@@ -332,8 +322,7 @@ serialise(const QVector<Kleo::DN::Attribute> &dn, const QString &sep)
     return result.join(sep);
 }
 
-static Kleo::DN::Attribute::List
-reorder_dn(const Kleo::DN::Attribute::List &dn)
+static Kleo::DN::Attribute::List reorder_dn(const Kleo::DN::Attribute::List &dn)
 {
     const QStringList &attrOrder = Kleo::DN::attributeOrder();
 
@@ -353,8 +342,7 @@ reorder_dn(const Kleo::DN::Attribute::List &dn)
     for (QStringList::const_iterator oit = attrOrder.begin(); oit != attrOrder.end(); ++oit) {
         if (*oit == QLatin1String("_X_")) {
             // insert the unknown attrs
-            std::copy(unknownEntries.begin(), unknownEntries.end(),
-                      std::back_inserter(result));
+            std::copy(unknownEntries.begin(), unknownEntries.end(), std::back_inserter(result));
             unknownEntries.clear(); // don't produce dup's
         } else {
             for (Kleo::DN::const_iterator dnit = dn.begin(); dnit != dn.end(); ++dnit) {
@@ -500,7 +488,7 @@ QString Kleo::DN::operator[](const QString &attr) const
         return QString();
     }
     const QString attrUpper = attr.toUpper();
-    for (QVector<Attribute>::const_iterator it = d->attributes.constBegin(); it != d->attributes.constEnd(); ++it) {
+    for (QList<Attribute>::const_iterator it = d->attributes.constBegin(); it != d->attributes.constEnd(); ++it) {
         if ((*it).name() == attrUpper) {
             return (*it).value();
         }
@@ -508,7 +496,7 @@ QString Kleo::DN::operator[](const QString &attr) const
     return QString();
 }
 
-static QVector<Kleo::DN::Attribute> empty;
+static QList<Kleo::DN::Attribute> empty;
 
 Kleo::DN::const_iterator Kleo::DN::begin() const
 {
@@ -525,25 +513,27 @@ Kleo::DN::const_iterator Kleo::DN::end() const
 namespace
 {
 static const QMap<QString, KLazyLocalizedString> attributeNamesAndLabels = {
-    { QStringLiteral("CN"), kli18n("Common name") },
-    { QStringLiteral("SN"), kli18n("Surname") },
-    { QStringLiteral("GN"), kli18n("Given name") },
-    { QStringLiteral("L"), kli18n("Location") },
-    { QStringLiteral("T"), kli18n("Title") },
-    { QStringLiteral("OU"), kli18n("Organizational unit") },
-    { QStringLiteral("O"), kli18n("Organization") },
-    { QStringLiteral("PC"), kli18n("Postal code") },
-    { QStringLiteral("C"), kli18n("Country code") },
-    { QStringLiteral("SP"), kli18n("State or province") },
-    { QStringLiteral("DC"), kli18n("Domain component") },
-    { QStringLiteral("BC"), kli18n("Business category") },
-    { QStringLiteral("EMAIL"), kli18n("Email address") },
-    { QStringLiteral("MAIL"), kli18n("Mail address") },
-    { QStringLiteral("MOBILE"), kli18n("Mobile phone number") },
-    { QStringLiteral("TEL"), kli18n("Telephone number") },
-    { QStringLiteral("FAX"), kli18n("Fax number") },
-    { QStringLiteral("STREET"), kli18n("Street address") },
-    { QStringLiteral("UID"), kli18n("Unique ID") }
+    // clang-format off
+    {QStringLiteral("CN"),     kli18n("Common name")        },
+    {QStringLiteral("SN"),     kli18n("Surname")            },
+    {QStringLiteral("GN"),     kli18n("Given name")         },
+    {QStringLiteral("L"),      kli18n("Location")           },
+    {QStringLiteral("T"),      kli18n("Title")              },
+    {QStringLiteral("OU"),     kli18n("Organizational unit")},
+    {QStringLiteral("O"),      kli18n("Organization")       },
+    {QStringLiteral("PC"),     kli18n("Postal code")        },
+    {QStringLiteral("C"),      kli18n("Country code")       },
+    {QStringLiteral("SP"),     kli18n("State or province")  },
+    {QStringLiteral("DC"),     kli18n("Domain component")   },
+    {QStringLiteral("BC"),     kli18n("Business category")  },
+    {QStringLiteral("EMAIL"),  kli18n("Email address")      },
+    {QStringLiteral("MAIL"),   kli18n("Mail address")       },
+    {QStringLiteral("MOBILE"), kli18n("Mobile phone number")},
+    {QStringLiteral("TEL"),    kli18n("Telephone number")   },
+    {QStringLiteral("FAX"),    kli18n("Fax number")         },
+    {QStringLiteral("STREET"), kli18n("Street address")     },
+    {QStringLiteral("UID"),    kli18n("Unique ID")          },
+    // clang-format on
 };
 }
 
