@@ -23,6 +23,7 @@
 #include <libkleo/filesystemwatcher.h>
 #include <libkleo/keygroup.h>
 #include <libkleo/keygroupconfig.h>
+#include <libkleo/keyhelpers.h>
 #include <libkleo/predicates.h>
 #include <libkleo/qtstlhelpers.h>
 #include <libkleo/stl_util.h>
@@ -550,13 +551,6 @@ void KeyCache::enableRemarks(bool value)
         if (d->m_initalized && !d->m_refreshJob) {
             qCDebug(LIBKLEO_LOG) << "Reloading keycache with remarks enabled";
             reload();
-        } else {
-            connect(d->m_refreshJob.data(), &RefreshKeysJob::done, this, [this](const GpgME::KeyListResult &) {
-                qCDebug(LIBKLEO_LOG) << "Reloading keycache with remarks enabled";
-                QTimer::singleShot(1s, this, [this]() {
-                    reload();
-                });
-            });
         }
     } else {
         d->m_remarks_enabled = value;
@@ -572,6 +566,16 @@ void KeyCache::Private::refreshJobDone(const KeyListResult &result)
 {
     m_refreshJob.clear();
     q->enableFileSystemWatcher(true);
+    if (!m_initalized && q->remarksEnabled()) {
+        // trigger another key listing to read signatures and signature notations
+        QMetaObject::invokeMethod(
+            q,
+            [this]() {
+                qCDebug(LIBKLEO_LOG) << "Reloading keycache with remarks enabled";
+                q->reload();
+            },
+            Qt::QueuedConnection);
+    }
     m_initalized = true;
     updateGroupCache();
     Q_EMIT q->keyListingDone(result);
@@ -803,17 +807,17 @@ namespace
 struct ready_for_signing {
     bool operator()(const Key &key) const
     {
-#if 1
         ACCEPT(hasSecret);
-        ACCEPT(canReallySign);
+#if GPGMEPP_KEY_HAS_HASCERTIFY_SIGN_ENCRYPT_AUTHENTICATE
+        ACCEPT(hasSign);
+#else
+        ACCEPT(canSign);
+#endif
         REJECT(isRevoked);
         REJECT(isExpired);
         REJECT(isDisabled);
         REJECT(isInvalid);
         return true;
-#else
-        return key.hasSecret() && key.canReallySign() && !key.isRevoked() && !key.isExpired() && !key.isDisabled() && !key.isInvalid();
-#endif
 #undef DO
     }
 };
@@ -828,14 +832,18 @@ struct ready_for_encryption {
     bool operator()(const Key &key) const
     {
 #if 1
+#if GPGMEPP_KEY_HAS_HASCERTIFY_SIGN_ENCRYPT_AUTHENTICATE
+        ACCEPT(hasEncrypt);
+#else
         ACCEPT(canEncrypt);
+#endif
         REJECT(isRevoked);
         REJECT(isExpired);
         REJECT(isDisabled);
         REJECT(isInvalid);
         return true;
 #else
-        return key.canEncrypt() && !key.isRevoked() && !key.isExpired() && !key.isDisabled() && !key.isInvalid();
+        return key.hasEncrypt() && !key.isRevoked() && !key.isExpired() && !key.isDisabled() && !key.isInvalid();
 #endif
     }
 #undef DO
@@ -1370,7 +1378,7 @@ public:
     void updateKeyCache();
 
     QPointer<KeyCache> m_cache;
-    QVector<QGpgME::ListAllKeysJob *> m_jobsPending;
+    QList<QGpgME::ListAllKeysJob *> m_jobsPending;
     std::vector<Key> m_keys;
     KeyListResult m_mergedResult;
     bool m_canceled;
@@ -1488,12 +1496,10 @@ Error KeyCache::RefreshKeysJob::Private::startKeyListing(GpgME::Protocol proto)
     if (!job) {
         return Error();
     }
-#ifdef QGPGME_LISTALLKEYSJOB_HAS_OPTIONS
     if (!m_cache->initialized()) {
         // avoid delays during the initial key listing
         job->setOptions(QGpgME::ListAllKeysJob::DisableAutomaticTrustDatabaseCheck);
     }
-#endif
 
 #if 0
     aheinecke: 2017.01.12:
@@ -1621,10 +1627,10 @@ GpgME::Key KeyCache::findBestByMailBox(const char *addr, GpgME::Protocol proto,
         if (proto != Protocol::UnknownProtocol && k.protocol() != proto) {
             continue;
         }
-        if (usage == KeyUsage::Encrypt && !k.canEncrypt()) {
+        if (usage == KeyUsage::Encrypt && !keyHasEncrypt(k)) {
             continue;
         }
-        if (usage == KeyUsage::Sign && (!k.canSign() || !k.hasSecret())) {
+        if (usage == KeyUsage::Sign && (!keyHasSign(k) || !k.hasSecret())) {
             continue;
         }
         const time_t creationTime = creationTimeOfNewestSuitableSubKey(k, usage);
@@ -1668,25 +1674,45 @@ bool allKeysAllowUsage(const T &keys, KeyCache::KeyUsage usage)
     case KeyCache::KeyUsage::AnyUsage:
         return true;
     case KeyCache::KeyUsage::Sign:
-        return std::all_of(std::begin(keys), std::end(keys), std::mem_fn(&Key::canSign));
+        return std::all_of(std::begin(keys),
+                           std::end(keys),
+#if GPGMEPP_KEY_HAS_HASCERTIFY_SIGN_ENCRYPT_AUTHENTICATE
+                           std::mem_fn(&Key::hasSign)
+#else
+                           Kleo::keyHasSign
+#endif
+        );
     case KeyCache::KeyUsage::Encrypt:
-        return std::all_of(std::begin(keys), std::end(keys), std::mem_fn(&Key::canEncrypt));
+        return std::all_of(std::begin(keys),
+                           std::end(keys),
+#if GPGMEPP_KEY_HAS_HASCERTIFY_SIGN_ENCRYPT_AUTHENTICATE
+                           std::mem_fn(&Key::hasEncrypt)
+#else
+                           Kleo::keyHasEncrypt
+#endif
+        );
     case KeyCache::KeyUsage::Certify:
-        return std::all_of(std::begin(keys), std::end(keys), std::mem_fn(&Key::canCertify));
+        return std::all_of(std::begin(keys),
+                           std::end(keys),
+#if GPGMEPP_KEY_HAS_HASCERTIFY_SIGN_ENCRYPT_AUTHENTICATE
+                           std::mem_fn(&Key::hasCertify)
+#else
+                           Kleo::keyHasCertify
+#endif
+        );
     case KeyCache::KeyUsage::Authenticate:
-        return std::all_of(std::begin(keys), std::end(keys), std::mem_fn(&Key::canAuthenticate));
+        return std::all_of(std::begin(keys),
+                           std::end(keys),
+#if GPGMEPP_KEY_HAS_HASCERTIFY_SIGN_ENCRYPT_AUTHENTICATE
+                           std::mem_fn(&Key::hasAuthenticate)
+#else
+                           Kleo::keyHasAuthenticate
+#endif
+        );
     }
     qCDebug(LIBKLEO_LOG) << __func__ << "called with invalid usage" << int(usage);
     return false;
 }
-
-template<typename T>
-bool allKeysHaveProtocol(const T &keys, Protocol protocol)
-{
-    return std::all_of(std::begin(keys), std::end(keys), [protocol](const auto &key) {
-        return key.protocol() == protocol;
-    });
-}
 }
 
 KeyGroup KeyCache::findGroup(const QString &name, Protocol protocol, KeyUsage usage) const
