@@ -7,34 +7,34 @@
     SPDX-License-Identifier: GPL-2.0-or-later
 */
 
+#include <config-libkleo.h>
+
 #include "keylistsortfilterproxymodel.h"
 
 #include "keylist.h"
 #include "keylistmodel.h"
-#include "kleo/keyfilter.h"
-#include "kleo/keygroup.h"
-#include "kleo/stl_util.h"
+
+#include <libkleo/keyfilter.h>
+#include <libkleo/keygroup.h>
+#include <libkleo/stl_util.h>
 
 #include <libkleo_debug.h>
 
 #include <gpgme++/key.h>
 
-#include <gpgme++/gpgmepp_version.h>
-#if GPGMEPP_VERSION >= 0x10E00 // 1.14.0
-# define GPGME_HAS_REMARKS
-#endif
-
 using namespace Kleo;
 using namespace GpgME;
 
 AbstractKeyListSortFilterProxyModel::AbstractKeyListSortFilterProxyModel(QObject *p)
-    : QSortFilterProxyModel(p), KeyListModelInterface()
+    : QSortFilterProxyModel(p)
+    , KeyListModelInterface()
 {
     init();
 }
 
 AbstractKeyListSortFilterProxyModel::AbstractKeyListSortFilterProxyModel(const AbstractKeyListSortFilterProxyModel &other)
-    : QSortFilterProxyModel(), KeyListModelInterface()
+    : QSortFilterProxyModel()
+    , KeyListModelInterface()
 {
     Q_UNUSED(other)
     init();
@@ -43,12 +43,14 @@ AbstractKeyListSortFilterProxyModel::AbstractKeyListSortFilterProxyModel(const A
 void AbstractKeyListSortFilterProxyModel::init()
 {
     setDynamicSortFilter(true);
-    setSortRole(Qt::EditRole);    // EditRole can be expected to be in a less formatted way, better for sorting
+    setSortRole(Qt::EditRole); // EditRole can be expected to be in a less formatted way, better for sorting
     setFilterRole(Qt::DisplayRole);
     setFilterCaseSensitivity(Qt::CaseInsensitive);
 }
 
-AbstractKeyListSortFilterProxyModel::~AbstractKeyListSortFilterProxyModel() {}
+AbstractKeyListSortFilterProxyModel::~AbstractKeyListSortFilterProxyModel()
+{
+}
 
 Key AbstractKeyListSortFilterProxyModel::key(const QModelIndex &idx) const
 {
@@ -68,7 +70,8 @@ std::vector<Key> AbstractKeyListSortFilterProxyModel::keys(const QList<QModelInd
     }
     QList<QModelIndex> mapped;
     mapped.reserve(indexes.size());
-    std::transform(indexes.begin(), indexes.end(),
+    std::transform(indexes.begin(), //
+                   indexes.end(),
                    std::back_inserter(mapped),
                    [this](const QModelIndex &idx) {
                        return mapToSource(idx);
@@ -98,7 +101,8 @@ QList<QModelIndex> AbstractKeyListSortFilterProxyModel::indexes(const std::vecto
         const QList<QModelIndex> source = klmi->indexes(keys);
         QList<QModelIndex> mapped;
         mapped.reserve(source.size());
-        std::transform(source.begin(), source.end(),
+        std::transform(source.begin(), //
+                       source.end(),
                        std::back_inserter(mapped),
                        [this](const QModelIndex &idx) {
                            return mapFromSource(idx);
@@ -108,7 +112,7 @@ QList<QModelIndex> AbstractKeyListSortFilterProxyModel::indexes(const std::vecto
     return QList<QModelIndex>();
 }
 
-QModelIndex AbstractKeyListSortFilterProxyModel::index(const Kleo::KeyGroup& group) const
+QModelIndex AbstractKeyListSortFilterProxyModel::index(const Kleo::KeyGroup &group) const
 {
     if (const KeyListModelInterface *const klmi = dynamic_cast<KeyListModelInterface *>(sourceModel())) {
         return mapFromSource(klmi->index(group));
@@ -119,28 +123,35 @@ QModelIndex AbstractKeyListSortFilterProxyModel::index(const Kleo::KeyGroup& gro
 class KeyListSortFilterProxyModel::Private
 {
     friend class ::Kleo::KeyListSortFilterProxyModel;
+
 public:
     explicit Private()
-        : keyFilter() {}
-    ~Private() {}
+        : keyFilter()
+    {
+    }
+    ~Private()
+    {
+    }
 
 private:
     std::shared_ptr<const KeyFilter> keyFilter;
 };
 
 KeyListSortFilterProxyModel::KeyListSortFilterProxyModel(QObject *p)
-    : AbstractKeyListSortFilterProxyModel(p), d(new Private)
+    : AbstractKeyListSortFilterProxyModel(p)
+    , d(new Private)
 {
-
 }
 
 KeyListSortFilterProxyModel::KeyListSortFilterProxyModel(const KeyListSortFilterProxyModel &other)
-    : AbstractKeyListSortFilterProxyModel(other), d(new Private(*other.d))
+    : AbstractKeyListSortFilterProxyModel(other)
+    , d(new Private(*other.d))
 {
-
 }
 
-KeyListSortFilterProxyModel::~KeyListSortFilterProxyModel() {}
+KeyListSortFilterProxyModel::~KeyListSortFilterProxyModel()
+{
+}
 
 KeyListSortFilterProxyModel *KeyListSortFilterProxyModel::clone() const
 {
@@ -167,17 +178,18 @@ bool KeyListSortFilterProxyModel::filterAcceptsRow(int source_row, const QModelI
     // 0. Keep parents of matching children:
     //
     const QModelIndex index = sourceModel()->index(source_row, 0, source_parent);
-    for (int i = 0, end = sourceModel()->rowCount(index); i != end; ++i)
+    for (int i = 0, end = sourceModel()->rowCount(index); i != end; ++i) {
         if (filterAcceptsRow(i, index)) {
             return true;
         }
+    }
 
     //
     // 1. Check filterRegExp
     //
     const int role = filterRole();
     const int col = filterKeyColumn();
-    const QRegExp rx = filterRegExp();
+    const QRegularExpression rx = filterRegularExpression();
     const QModelIndex nameIndex = sourceModel()->index(source_row, KeyList::PrettyName, source_parent);
 
     const KeyListModelInterface *const klm = dynamic_cast<KeyListModelInterface *>(sourceModel());
@@ -195,13 +207,12 @@ bool KeyListSortFilterProxyModel::filterAcceptsRow(int source_row, const QModelI
     } else if (!key.isNull()) {
         // By default match against the full uid data (name / email / comment / dn)
         bool match = false;
-        for (const auto &uid: key.userIDs()) {
+        for (const auto &uid : key.userIDs()) {
             const auto id = QString::fromUtf8(uid.id());
             if (id.contains(rx)) {
                 match = true;
                 break;
             }
-#ifdef GPGME_HAS_REMARKS
             // Also match against remarks (search tags)
             const auto alm = dynamic_cast<AbstractKeyListModel *>(sourceModel());
             if (alm) {
@@ -211,7 +222,14 @@ bool KeyListSortFilterProxyModel::filterAcceptsRow(int source_row, const QModelI
                     break;
                 }
             }
-#endif
+            // Also match against fingerprints
+            for (const auto &subkey : key.subkeys()) {
+                const auto fpr = QString::fromLatin1(subkey.fingerprint());
+                if (fpr.contains(rx)) {
+                    match = true;
+                    break;
+                }
+            }
         }
         if (!match) {
             return false;
@@ -227,7 +245,7 @@ bool KeyListSortFilterProxyModel::filterAcceptsRow(int source_row, const QModelI
     //
     // 2. For keys check that key filters match (if any are defined)
     //
-    if (d->keyFilter && !key.isNull()) {   // avoid artifacts when no filters are defined
+    if (d->keyFilter && !key.isNull()) { // avoid artifacts when no filters are defined
         return d->keyFilter->matches(key, KeyFilter::Filtering);
     }
 
@@ -235,3 +253,4 @@ bool KeyListSortFilterProxyModel::filterAcceptsRow(int source_row, const QModelI
     return true;
 }
 
+#include "moc_keylistsortfilterproxymodel.cpp"
