@@ -10,7 +10,8 @@
 
 #include "auditlogviewer.h"
 
-#include <kleo/auditlogentry.h>
+#include <libkleo/auditlogentry.h>
+#include <libkleo/formatting.h>
 
 #include <KConfigGroup>
 #include <KGuiItem>
@@ -20,7 +21,7 @@
 #include <KStandardGuiItem>
 
 #ifdef HAVE_PIMTEXTEDIT
-#include <KPIMTextEdit/RichTextEditor>
+#include <TextCustomEditor/RichTextEditor>
 #else
 #include <QTextEdit>
 #endif
@@ -43,7 +44,7 @@ AuditLogViewer::AuditLogViewer(const QString &log, QWidget *parent)
     , m_log(/* sic */)
     ,
 #ifdef HAVE_PIMTEXTEDIT
-    m_textEdit(new KPIMTextEdit::RichTextEditorWidget(this))
+    m_textEdit(new TextCustomEditor::RichTextEditorWidget(this))
 #else
     m_textEdit(new QTextEdit(this))
 #endif
@@ -103,7 +104,7 @@ void AuditLogViewer::showAuditLog(QWidget *parent, const AuditLogEntry &auditLog
     }
     if (err && err.code() != GPG_ERR_NO_DATA) {
         KMessageBox::information(parent,
-                                 i18n("An error occurred while trying to retrieve the GnuPG Audit Log:\n%1", QString::fromLocal8Bit(err.asString())),
+                                 i18n("An error occurred while trying to retrieve the GnuPG Audit Log:\n%1", Formatting::errorAsString(err)),
                                  i18n("GnuPG Audit Log Error"));
         return;
     }
@@ -167,7 +168,7 @@ void AuditLogViewer::slotCopyClip()
 
 void AuditLogViewer::readConfig()
 {
-    KConfigGroup group(KSharedConfig::openConfig(), "AuditLogViewer");
+    KConfigGroup group(KSharedConfig::openConfig(), QStringLiteral("AuditLogViewer"));
     const QSize size = group.readEntry("Size", QSize());
     if (size.isValid()) {
         resize(size);
@@ -178,7 +179,9 @@ void AuditLogViewer::readConfig()
 
 void AuditLogViewer::writeConfig()
 {
-    KConfigGroup group(KSharedConfig::openConfig(), "AuditLogViewer");
+    KConfigGroup group(KSharedConfig::openConfig(), QStringLiteral("AuditLogViewer"));
     group.writeEntry("Size", size());
     group.sync();
 }
+
+#include "moc_auditlogviewer.cpp"
