@@ -22,9 +22,9 @@
 
 #include <KLocalizedString>
 
+#include <QList>
 #include <QSortFilterProxyModel>
 #include <QTimer>
-#include <QVector>
 
 #include <gpgme++/key.h>
 
@@ -68,30 +68,35 @@ private:
     QString mFingerprint;
 };
 
-class ProxyModel : public QSortFilterProxyModel
+static QString formatUserID(const GpgME::Key &key)
+{
+    const auto userID = key.userID(0);
+    QString name;
+    QString email;
+
+    if (key.protocol() == GpgME::OpenPGP) {
+        name = QString::fromUtf8(userID.name());
+        email = QString::fromUtf8(userID.email());
+    } else {
+        const Kleo::DN dn(userID.id());
+        name = dn[QStringLiteral("CN")];
+        email = dn[QStringLiteral("EMAIL")];
+    }
+    return email.isEmpty() ? name : name.isEmpty() ? email : i18nc("Name <email>", "%1 <%2>", name, email);
+}
+
+class SortAndFormatCertificatesProxyModel : public QSortFilterProxyModel
 {
     Q_OBJECT
 
-private:
-    struct CustomItem {
-        QIcon icon;
-        QString text;
-        QVariant data;
-        QString toolTip;
-    };
-
 public:
-    ProxyModel(QObject *parent = nullptr)
-        : QSortFilterProxyModel(parent)
+    SortAndFormatCertificatesProxyModel(KeyUsage::Flags usageFlags, QObject *parent = nullptr)
+        : QSortFilterProxyModel{parent}
+        , mIconProvider{usageFlags}
     {
     }
 
-    ~ProxyModel() override
-    {
-        qDeleteAll(mFrontItems);
-        qDeleteAll(mBackItems);
-    }
-
+private:
     bool lessThan(const QModelIndex &left, const QModelIndex &right) const override
     {
         const auto leftKey = sourceModel()->data(left, KeyList::KeyRole).value<GpgME::Key>();
@@ -111,34 +116,114 @@ public:
         if (rUid.isNull()) {
             return true;
         }
-        int cmp = strcmp(lUid.id(), rUid.id());
+        const auto leftNameAndEmail = formatUserID(leftKey);
+        const auto rightNameAndEmail = formatUserID(rightKey);
+        const int cmp = QString::localeAwareCompare(leftNameAndEmail, rightNameAndEmail);
         if (cmp) {
             return cmp < 0;
         }
 
-        if (lUid.validity() == rUid.validity()) {
-            /* Both are the same check which one is newer. */
-            time_t oldTime = 0;
-            for (const GpgME::Subkey &s : leftKey.subkeys()) {
-                if (s.isRevoked() || s.isInvalid() || s.isDisabled()) {
-                    continue;
-                }
-                if (s.creationTime() > oldTime) {
-                    oldTime = s.creationTime();
-                }
-            }
-            time_t newTime = 0;
-            for (const GpgME::Subkey &s : rightKey.subkeys()) {
-                if (s.isRevoked() || s.isInvalid() || s.isDisabled()) {
-                    continue;
-                }
-                if (s.creationTime() > newTime) {
-                    newTime = s.creationTime();
-                }
-            }
-            return newTime < oldTime;
+        if (lUid.validity() != rUid.validity()) {
+            return lUid.validity() > rUid.validity();
         }
-        return lUid.validity() > rUid.validity();
+
+        /* Both have the same validity, check which one is newer. */
+        time_t leftTime = 0;
+        for (const GpgME::Subkey &s : leftKey.subkeys()) {
+            if (s.isBad()) {
+                continue;
+            }
+            if (s.creationTime() > leftTime) {
+                leftTime = s.creationTime();
+            }
+        }
+        time_t rightTime = 0;
+        for (const GpgME::Subkey &s : rightKey.subkeys()) {
+            if (s.isBad()) {
+                continue;
+            }
+            if (s.creationTime() > rightTime) {
+                rightTime = s.creationTime();
+            }
+        }
+        if (rightTime != leftTime) {
+            return leftTime > rightTime;
+        }
+
+        // as final resort we compare the fingerprints
+        return strcmp(leftKey.primaryFingerprint(), rightKey.primaryFingerprint()) < 0;
+    }
+
+protected:
+    QVariant data(const QModelIndex &index, int role) const override
+    {
+        if (!index.isValid()) {
+            return QVariant();
+        }
+
+        const auto key = QSortFilterProxyModel::data(index, KeyList::KeyRole).value<GpgME::Key>();
+        Q_ASSERT(!key.isNull());
+        if (key.isNull()) {
+            return QVariant();
+        }
+
+        switch (role) {
+        case Qt::DisplayRole:
+        case Qt::AccessibleTextRole: {
+            const auto nameAndEmail = formatUserID(key);
+            if (Kleo::KeyCache::instance()->pgpOnly()) {
+                return i18nc("Name <email> (validity, created: date)",
+                             "%1 (%2, created: %3)",
+                             nameAndEmail,
+                             Kleo::Formatting::complianceStringShort(key),
+                             Kleo::Formatting::creationDateString(key));
+            } else {
+                return i18nc("Name <email> (validity, type, created: date)",
+                             "%1 (%2, %3, created: %4)",
+                             nameAndEmail,
+                             Kleo::Formatting::complianceStringShort(key),
+                             Formatting::displayName(key.protocol()),
+                             Kleo::Formatting::creationDateString(key));
+            }
+        }
+        case Qt::ToolTipRole: {
+            using namespace Kleo::Formatting;
+            return Kleo::Formatting::toolTip(key, Validity | Issuer | Subject | Fingerprint | ExpiryDates | UserIDs);
+        }
+        case Qt::DecorationRole: {
+            return mIconProvider.icon(key);
+        }
+        default:
+            return QSortFilterProxyModel::data(index, role);
+        }
+    }
+
+private:
+    Formatting::IconProvider mIconProvider;
+};
+
+class CustomItemsProxyModel : public QSortFilterProxyModel
+{
+    Q_OBJECT
+
+private:
+    struct CustomItem {
+        QIcon icon;
+        QString text;
+        QVariant data;
+        QString toolTip;
+    };
+
+public:
+    CustomItemsProxyModel(QObject *parent = nullptr)
+        : QSortFilterProxyModel(parent)
+    {
+    }
+
+    ~CustomItemsProxyModel() override
+    {
+        qDeleteAll(mFrontItems);
+        qDeleteAll(mBackItems);
     }
 
     bool isCustomItem(const int row) const
@@ -197,13 +282,14 @@ public:
 
     QModelIndex mapToSource(const QModelIndex &index) const override
     {
-        if (!isCustomItem(index.row())) {
-            const int row = index.row() - mFrontItems.count();
-
-            return sourceModel()->index(row, index.column());
-        } else {
+        if (!index.isValid()) {
             return {};
         }
+        if (!isCustomItem(index.row())) {
+            const int sourceRow = index.row() - mFrontItems.count();
+            return QSortFilterProxyModel::mapToSource(createIndex(sourceRow, index.column(), index.internalPointer()));
+        }
+        return {};
     }
 
     QModelIndex mapFromSource(const QModelIndex &source_index) const override
@@ -262,57 +348,12 @@ public:
             }
         }
 
-        const auto key = QSortFilterProxyModel::data(index, KeyList::KeyRole).value<GpgME::Key>();
-        Q_ASSERT(!key.isNull());
-        if (key.isNull()) {
-            return QVariant();
-        }
-
-        switch (role) {
-        case Qt::DisplayRole:
-        case Qt::AccessibleTextRole: {
-            const auto userID = key.userID(0);
-            QString name;
-            QString email;
-
-            if (key.protocol() == GpgME::OpenPGP) {
-                name = QString::fromUtf8(userID.name());
-                email = QString::fromUtf8(userID.email());
-            } else {
-                const Kleo::DN dn(userID.id());
-                name = dn[QStringLiteral("CN")];
-                email = dn[QStringLiteral("EMAIL")];
-            }
-            const auto nameAndEmail = email.isEmpty() ? name : name.isEmpty() ? email : i18nc("Name <email>", "%1 <%2>", name, email);
-            if (Kleo::KeyCache::instance()->pgpOnly()) {
-                return i18nc("Name <email> (validity, created: date)",
-                             "%1 (%2, created: %3)",
-                             nameAndEmail,
-                             Kleo::Formatting::complianceStringShort(key),
-                             Kleo::Formatting::creationDateString(key));
-            } else {
-                return i18nc("Name <email> (validity, type, created: date)",
-                             "%1 (%2, %3, created: %4)",
-                             nameAndEmail,
-                             Kleo::Formatting::complianceStringShort(key),
-                             Formatting::displayName(key.protocol()),
-                             Kleo::Formatting::creationDateString(key));
-            }
-        }
-        case Qt::ToolTipRole: {
-            using namespace Kleo::Formatting;
-            return Kleo::Formatting::toolTip(key, Validity | Issuer | Subject | Fingerprint | ExpiryDates | UserIDs);
-        }
-        case Qt::DecorationRole:
-            return Kleo::Formatting::iconForUid(key.userID(0));
-        default:
-            return QSortFilterProxyModel::data(index, role);
-        }
+        return QSortFilterProxyModel::data(index, role);
     }
 
 private:
-    QVector<CustomItem *> mFrontItems;
-    QVector<CustomItem *> mBackItems;
+    QList<CustomItem *> mFrontItems;
+    QList<CustomItem *> mBackItems;
 };
 
 } // anonymous namespace
@@ -322,9 +363,11 @@ namespace Kleo
 class KeySelectionComboPrivate
 {
 public:
-    KeySelectionComboPrivate(KeySelectionCombo *parent)
+    KeySelectionComboPrivate(KeySelectionCombo *parent, bool secretOnly_, KeyUsage::Flags usage)
         : wasEnabled(true)
-        , q(parent)
+        , secretOnly{secretOnly_}
+        , usageFlags{usage}
+        , q{parent}
     {
     }
 
@@ -420,7 +463,8 @@ public:
 
     Kleo::AbstractKeyListModel *model = nullptr;
     SortFilterProxyModel *sortFilterProxy = nullptr;
-    ProxyModel *proxyModel = nullptr;
+    SortAndFormatCertificatesProxyModel *sortAndFormatProxy = nullptr;
+    CustomItemsProxyModel *proxyModel = nullptr;
     std::shared_ptr<Kleo::KeyCache> cache;
     QMap<GpgME::Protocol, QString> defaultKeys;
     bool wasEnabled = false;
@@ -430,6 +474,7 @@ public:
     QString mPerfectMatchMbox;
     GpgME::Key keyBeforeModelChange;
     QVariant customItemBeforeModelChange;
+    KeyUsage::Flags usageFlags;
 
 private:
     KeySelectionCombo *const q;
@@ -440,28 +485,47 @@ private:
 using namespace Kleo;
 
 KeySelectionCombo::KeySelectionCombo(QWidget *parent)
-    : KeySelectionCombo(true, parent)
+    : KeySelectionCombo(true, KeyUsage::None, parent)
 {
 }
 
 KeySelectionCombo::KeySelectionCombo(bool secretOnly, QWidget *parent)
+    : KeySelectionCombo(secretOnly, KeyUsage::None, parent)
+{
+}
+
+KeySelectionCombo::KeySelectionCombo(KeyUsage::Flags usage, QWidget *parent)
+    : KeySelectionCombo{false, usage, parent}
+{
+}
+
+KeySelectionCombo::KeySelectionCombo(KeyUsage::Flag usage, QWidget *parent)
+    : KeySelectionCombo{false, usage, parent}
+{
+}
+
+KeySelectionCombo::KeySelectionCombo(bool secretOnly, KeyUsage::Flags usage, QWidget *parent)
     : QComboBox(parent)
-    , d(new KeySelectionComboPrivate(this))
+    , d(new KeySelectionComboPrivate(this, secretOnly, usage))
 {
     // set a non-empty string as accessible description to prevent screen readers
     // from reading the tool tip which isn't meant for screen readers
     setAccessibleDescription(QStringLiteral(" "));
     d->model = Kleo::AbstractKeyListModel::createFlatKeyListModel(this);
-    d->secretOnly = secretOnly;
 
     d->sortFilterProxy = new SortFilterProxyModel(this);
     d->sortFilterProxy->setSourceModel(d->model);
 
-    d->proxyModel = new ProxyModel(this);
-    d->proxyModel->setSourceModel(d->sortFilterProxy);
+    d->sortAndFormatProxy = new SortAndFormatCertificatesProxyModel{usage, this};
+    d->sortAndFormatProxy->setSourceModel(d->sortFilterProxy);
+    // initialize dynamic sorting
+    d->sortAndFormatProxy->sort(0);
+
+    d->proxyModel = new CustomItemsProxyModel{this};
+    d->proxyModel->setSourceModel(d->sortAndFormatProxy);
 
     setModel(d->proxyModel);
-    connect(this, qOverload<int>(&QComboBox::currentIndexChanged), this, [this](int row) {
+    connect(this, &QComboBox::currentIndexChanged, this, [this](int row) {
         if (row >= 0 && row < d->proxyModel->rowCount()) {
             if (d->proxyModel->isCustomItem(row)) {
                 Q_EMIT customItemSelected(currentData(Qt::UserRole));
@@ -536,7 +600,7 @@ void KeySelectionCombo::init()
         Q_EMIT keyListingFinished();
     }
 
-    connect(this, qOverload<int>(&QComboBox::currentIndexChanged), this, [this]() {
+    connect(this, &QComboBox::currentIndexChanged, this, [this]() {
         setToolTip(currentData(Qt::ToolTipRole).toString());
     });
 }
@@ -544,7 +608,6 @@ void KeySelectionCombo::init()
 void KeySelectionCombo::setKeyFilter(const std::shared_ptr<const KeyFilter> &kf)
 {
     d->sortFilterProxy->setKeyFilter(kf);
-    d->proxyModel->sort(0);
     d->updateWithDefaultKey();
 }
 
@@ -657,3 +720,5 @@ QString Kleo::KeySelectionCombo::defaultKey() const
     return defaultKey(GpgME::UnknownProtocol);
 }
 #include "keyselectioncombo.moc"
+
+#include "moc_keyselectioncombo.cpp"
