@@ -18,6 +18,7 @@
 #include "keylistview.h"
 #include "progressdialog.h"
 
+#include <libkleo/compat.h>
 #include <libkleo/compliance.h>
 #include <libkleo/dn.h>
 #include <libkleo/formatting.h>
@@ -89,22 +90,22 @@ static bool checkKeyUsage(const GpgME::Key &key, unsigned int keyUsage, QString
         }
     }
 
-    if (keyUsage & KeySelectionDialog::EncryptionKeys && !key.canEncrypt()) {
+    if (keyUsage & KeySelectionDialog::EncryptionKeys && !Kleo::keyHasEncrypt(key)) {
         qCDebug(KLEO_UI_LOG) << "key can't encrypt";
         setStatusString(i18n("The key is not designated for encryption."));
         return false;
     }
-    if (keyUsage & KeySelectionDialog::SigningKeys && !key.canSign()) {
+    if (keyUsage & KeySelectionDialog::SigningKeys && !Kleo::keyHasSign(key)) {
         qCDebug(KLEO_UI_LOG) << "key can't sign";
         setStatusString(i18n("The key is not designated for signing."));
         return false;
     }
-    if (keyUsage & KeySelectionDialog::CertificationKeys && !key.canCertify()) {
+    if (keyUsage & KeySelectionDialog::CertificationKeys && !Kleo::keyHasCertify(key)) {
         qCDebug(KLEO_UI_LOG) << "key can't certify";
         setStatusString(i18n("The key is not designated for certifying."));
         return false;
     }
-    if (keyUsage & KeySelectionDialog::AuthenticationKeys && !key.canAuthenticate()) {
+    if (keyUsage & KeySelectionDialog::AuthenticationKeys && !Kleo::keyHasAuthenticate(key)) {
         qCDebug(KLEO_UI_LOG) << "key can't authenticate";
         setStatusString(i18n("The key is not designated for authentication."));
         return false;
@@ -400,7 +401,7 @@ void KeySelectionDialog::setUpUI(Options options, const QString &initialQuery)
     QDialogButtonBox *buttonBox = new QDialogButtonBox(QDialogButtonBox::Ok | QDialogButtonBox::Cancel, this);
     mOkButton = buttonBox->button(QDialogButtonBox::Ok);
     mOkButton->setDefault(true);
-    mOkButton->setShortcut(Qt::CTRL | Qt::Key_Return);
+    mOkButton->setShortcut(QKeySequence(Qt::CTRL | Qt::Key_Return));
 
     mCheckSelectionTimer = new QTimer(this);
     mStartSearchTimer = new QTimer(this);
@@ -496,7 +497,7 @@ void KeySelectionDialog::setUpUI(Options options, const QString &initialQuery)
 
     if (qApp) {
         QSize dialogSize(sizeHint());
-        KConfigGroup dialogConfig(KSharedConfig::openStateConfig(), "Key Selection Dialog");
+        KConfigGroup dialogConfig(KSharedConfig::openStateConfig(), QStringLiteral("Key Selection Dialog"));
         dialogSize = dialogConfig.readEntry("Dialog size", dialogSize);
         const QByteArray headerState = dialogConfig.readEntry("header", QByteArray());
         if (!headerState.isEmpty()) {
@@ -528,7 +529,7 @@ void KeySelectionDialog::init(bool rememberChoice, bool extendedSelection, const
 KeySelectionDialog::~KeySelectionDialog()
 {
     disconnectSignals();
-    KConfigGroup dialogConfig(KSharedConfig::openStateConfig(), "Key Selection Dialog");
+    KConfigGroup dialogConfig(KSharedConfig::openStateConfig(), QStringLiteral("Key Selection Dialog"));
     dialogConfig.writeEntry("Dialog size", size());
     dialogConfig.writeEntry("header", mKeyListView->header()->saveState());
     dialogConfig.sync();
@@ -679,7 +680,7 @@ static void showKeyListError(QWidget *parent, const GpgME::Error &err)
         "<qt><p>An error occurred while fetching "
         "the keys from the backend:</p>"
         "<p><b>%1</b></p></qt>",
-        QString::fromLocal8Bit(err.asString()));
+        Formatting::errorAsString(err));
 
     KMessageBox::error(parent, msg, i18n("Key Listing Failed"));
 }
@@ -1017,3 +1018,5 @@ void KeySelectionDialog::showAllItems()
         item->setHidden(false);
     }
 }
+
+#include "moc_keyselectiondialog.cpp"
