@@ -16,11 +16,13 @@
 #include "keyselectioncombo.h"
 #include "progressdialog.h"
 
+#include <libkleo/algorithm.h>
 #include <libkleo/compliance.h>
 #include <libkleo/debug.h>
 #include <libkleo/defaultkeyfilter.h>
 #include <libkleo/formatting.h>
 #include <libkleo/gnupg.h>
+#include <libkleo/keyhelpers.h>
 #include <libkleo/systeminfo.h>
 
 #include <libkleo_debug.h>
@@ -60,7 +62,7 @@ public:
     EncryptFilter()
         : DefaultKeyFilter()
     {
-        setCanEncrypt(DefaultKeyFilter::Set);
+        setHasEncrypt(DefaultKeyFilter::Set);
     }
 };
 static std::shared_ptr<KeyFilter> s_encryptFilter = std::shared_ptr<KeyFilter>(new EncryptFilter);
@@ -72,7 +74,7 @@ public:
         : DefaultKeyFilter()
     {
         setIsOpenPGP(DefaultKeyFilter::Set);
-        setCanEncrypt(DefaultKeyFilter::Set);
+        setHasEncrypt(DefaultKeyFilter::Set);
     }
 };
 static std::shared_ptr<KeyFilter> s_pgpEncryptFilter = std::shared_ptr<KeyFilter>(new OpenPGPFilter);
@@ -101,7 +103,7 @@ public:
         : DefaultKeyFilter()
     {
         setIsOpenPGP(DefaultKeyFilter::NotSet);
-        setCanEncrypt(DefaultKeyFilter::Set);
+        setHasEncrypt(DefaultKeyFilter::Set);
     }
 };
 static std::shared_ptr<KeyFilter> s_smimeEncryptFilter = std::shared_ptr<KeyFilter>(new SMIMEFilter);
@@ -219,13 +221,6 @@ static bool key_has_addr(const GpgME::Key &key, const QString &addr)
     return false;
 }
 
-bool anyKeyHasProtocol(const std::vector<Key> &keys, GpgME::Protocol protocol)
-{
-    return std::any_of(std::begin(keys), std::end(keys), [protocol](const auto &key) {
-        return key.protocol() == protocol;
-    });
-}
-
 Key findfirstKeyOfType(const std::vector<Key> &keys, GpgME::Protocol protocol)
 {
     const auto it = std::find_if(std::begin(keys), std::end(keys), [protocol](const auto &key) {
@@ -319,6 +314,10 @@ public:
         mFormatBtns->addButton(smimeBtn, SMIMEButtonId);
         mFormatBtns->setExclusive(!mAllowMixed);
 
+        connect(mFormatBtns, QOverload<QAbstractButton *>::of(&QButtonGroup::buttonClicked), q, [this]() {
+            updateOkButton();
+        });
+
         fmtLayout->addStretch(-1);
         fmtLayout->addWidget(pgpBtn);
         fmtLayout->addWidget(smimeBtn);
@@ -425,7 +424,7 @@ public:
     void checkAccepted()
     {
         if (mLastError || mLastError.isCanceled()) {
-            KMessageBox::error(q, QString::fromLocal8Bit(mLastError.asString()), i18n("Operation Failed"));
+            KMessageBox::error(q, Formatting::errorAsString(mLastError), i18n("Operation Failed"));
             mRunningJobs.clear();
             return;
         }
@@ -522,7 +521,7 @@ public:
         Q_ASSERT(!key.isNull() || protocol != UnknownProtocol);
         protocol = !key.isNull() ? key.protocol() : protocol;
 
-        auto combo = new KeySelectionCombo();
+        auto combo = new KeySelectionCombo{true, KeyUsage::Sign};
         auto comboWidget = new ComboWidget(combo);
 #ifndef NDEBUG
         combo->setObjectName(QStringLiteral("signing key"));
@@ -542,7 +541,7 @@ public:
         if (key.isNull() && protocol == OpenPGP) {
             combo->appendCustomItem(QIcon::fromTheme(QStringLiteral("document-new")), i18n("Generate a new key pair"), GenerateKey, mGenerateTooltip);
         }
-        combo->appendCustomItem(QIcon::fromTheme(QStringLiteral("emblem-unavailable")),
+        combo->appendCustomItem(Formatting::unavailableIcon(),
                                 i18n("Don't confirm identity and integrity"),
                                 IgnoreKey,
                                 i18nc("@info:tooltip for not selecting a key for signing.", "The E-Mail will not be cryptographically signed."));
@@ -615,7 +614,7 @@ public:
 
     ComboWidget *createEncryptionCombo(const QString &addr, const GpgME::Key &key, GpgME::Protocol fixedProtocol)
     {
-        auto combo = new KeySelectionCombo(false);
+        auto combo = new KeySelectionCombo{false, KeyUsage::Encrypt};
         auto comboWidget = new ComboWidget(combo);
 #ifndef NDEBUG
         combo->setObjectName(QStringLiteral("encryption key"));
@@ -639,7 +638,7 @@ public:
             combo->appendCustomItem(QIcon::fromTheme(QStringLiteral("document-new")), i18n("Generate a new key pair"), GenerateKey, mGenerateTooltip);
         }
 
-        combo->appendCustomItem(QIcon::fromTheme(QStringLiteral("emblem-unavailable")),
+        combo->appendCustomItem(Formatting::unavailableIcon(),
                                 i18n("No key. Recipient will be unable to decrypt."),
                                 IgnoreKey,
                                 i18nc("@info:tooltip for No Key selected for a specific recipient.",
@@ -803,13 +802,17 @@ public:
     {
         static QString origOkText = mOkButton->text();
         const bool isGenerate = bool(findVisibleKeySelectionComboWithGenerateKey());
-        const bool allVisibleEncryptionKeysAreIgnored = std::all_of(std::begin(mEncCombos), std::end(mEncCombos), [](auto combo) {
+        const bool allVisibleEncryptionKeysAreIgnored = Kleo::all_of(mEncCombos, [](auto combo) {
             return !combo->isVisible() || combo->currentData(Qt::UserRole).toInt() == IgnoreKey;
         });
+        const bool allVisibleEncryptionKeysAreUsable = Kleo::all_of(mEncCombos, [](auto combo) {
+            return !combo->isVisible() || combo->currentKey().isNull() || Kleo::canBeUsedForEncryption(combo->currentKey());
+        });
 
-        // If we don't encrypt the ok button is always enabled. But otherwise
+        // If we don't encrypt, then the OK button is always enabled. Likewise,
+        // if the "generate key" option is selected. Otherwise,
         // we only enable it if we encrypt to at least one recipient.
-        mOkButton->setEnabled(!mEncrypt || !allVisibleEncryptionKeysAreIgnored);
+        mOkButton->setEnabled(isGenerate || !mEncrypt || (!allVisibleEncryptionKeysAreIgnored && allVisibleEncryptionKeysAreUsable));
 
         mOkButton->setText(isGenerate ? i18n("Generate") : origOkText);
 
@@ -834,7 +837,7 @@ public:
                 if (protocol != GpgME::UnknownProtocol && key.protocol() != protocol) {
                     continue;
                 }
-                if (!Formatting::isKeyDeVs(key) || !Formatting::uidsHaveFullValidity(key)) {
+                if (!DeVSCompliance::keyIsCompliant(key)) {
                     de_vs = false;
                     break;
                 }
@@ -904,3 +907,5 @@ KeyResolver::Solution NewKeyApprovalDialog::result()
 }
 
 #include "newkeyapprovaldialog.moc"
+
+#include "moc_newkeyapprovaldialog.cpp"
