@@ -13,6 +13,8 @@
 
 #include "formatting.h"
 
+#include "algorithm.h"
+#include "compat.h"
 #include "compliance.h"
 #include "cryptoconfig.h"
 #include "gnupg.h"
@@ -41,6 +43,90 @@
 using namespace GpgME;
 using namespace Kleo;
 
+namespace
+{
+QIcon iconForValidityAndCompliance(UserID::Validity validity, bool isCompliant)
+{
+    switch (validity) {
+    case UserID::Ultimate:
+    case UserID::Full:
+    case UserID::Marginal:
+        return isCompliant ? Formatting::successIcon() : Formatting::infoIcon();
+    case UserID::Never:
+        return Formatting::errorIcon();
+    case UserID::Undefined:
+    case UserID::Unknown:
+    default:
+        return Formatting::infoIcon();
+    }
+}
+QIcon iconForValidity(const UserID &userId)
+{
+    const bool keyIsCompliant = !DeVSCompliance::isActive() || //
+        (DeVSCompliance::isCompliant() && DeVSCompliance::keyIsCompliant(userId.parent()));
+    return iconForValidityAndCompliance(userId.validity(), keyIsCompliant);
+}
+}
+
+QIcon Formatting::IconProvider::icon(const GpgME::Key &key) const
+{
+    if (usage.canEncrypt() && !Kleo::canBeUsedForEncryption(key)) {
+        return Formatting::errorIcon();
+    }
+    if (usage.canSign() && !Kleo::canBeUsedForSigning(key)) {
+        return Formatting::errorIcon();
+    }
+    if (key.isBad()) {
+        return Formatting::errorIcon();
+    }
+    const auto primaryUserId = key.userID(0);
+    if (Kleo::isRevokedOrExpired(primaryUserId)) {
+        return Formatting::errorIcon();
+    }
+    return iconForValidity(primaryUserId);
+}
+
+QIcon Formatting::IconProvider::icon(const KeyGroup &group) const
+{
+    if (usage.canEncrypt() && !Kleo::all_of(group.keys(), Kleo::canBeUsedForEncryption)) {
+        return Formatting::errorIcon();
+    }
+    if (usage.canSign() && !Kleo::all_of(group.keys(), Kleo::canBeUsedForSigning)) {
+        return Formatting::errorIcon();
+    }
+    return validityIcon(group);
+}
+
+QIcon Formatting::successIcon()
+{
+    return QIcon::fromTheme(QStringLiteral("emblem-success"));
+}
+
+QIcon Formatting::infoIcon()
+{
+    return QIcon::fromTheme(QStringLiteral("emblem-information"));
+}
+
+QIcon Formatting::questionIcon()
+{
+    return QIcon::fromTheme(QStringLiteral("emblem-question"));
+}
+
+QIcon Formatting::unavailableIcon()
+{
+    return QIcon::fromTheme(QStringLiteral("emblem-unavailable"));
+}
+
+QIcon Formatting::warningIcon()
+{
+    return QIcon::fromTheme(QStringLiteral("emblem-warning"));
+}
+
+QIcon Formatting::errorIcon()
+{
+    return QIcon::fromTheme(QStringLiteral("emblem-error"));
+}
+
 //
 // Name
 //
@@ -251,20 +337,20 @@ QString format_subkeytype(const Subkey &subkey)
 QString format_keyusage(const Key &key)
 {
     QStringList capabilities;
-    if (key.canReallySign()) {
+    if (Kleo::keyHasSign(key)) {
         if (key.isQualified()) {
             capabilities.push_back(i18n("Signing (Qualified)"));
         } else {
             capabilities.push_back(i18n("Signing"));
         }
     }
-    if (key.canEncrypt()) {
+    if (Kleo::keyHasEncrypt(key)) {
         capabilities.push_back(i18n("Encryption"));
     }
-    if (key.canCertify()) {
+    if (Kleo::keyHasCertify(key)) {
         capabilities.push_back(i18n("Certifying User-IDs"));
     }
-    if (key.canAuthenticate()) {
+    if (Kleo::keyHasAuthenticate(key)) {
         capabilities.push_back(i18n("SSH Authentication"));
     }
     return capabilities.join(QLatin1String(", "));
@@ -458,7 +544,7 @@ QString getValidityStatement(const Container &keys)
         if (someKeysAreBad) {
             return i18n("Some keys are revoked, expired, disabled, or invalid.");
         } else {
-            const bool allKeysAreFullyValid = std::all_of(keys.cbegin(), keys.cend(), &Formatting::uidsHaveFullValidity);
+            const bool allKeysAreFullyValid = std::all_of(keys.cbegin(), keys.cend(), &Kleo::allUserIDsHaveFullValidity);
             if (allKeysAreFullyValid) {
                 return i18n("All keys are certified.");
             } else {
@@ -801,7 +887,7 @@ QString Formatting::validityShort(const UserID::Signature &sig)
                 return i18n("class %1", sig.certClass());
             }
         }
-        Q_FALLTHROUGH();
+        [[fallthrough]];
         // fall through:
     case UserID::Signature::GeneralError:
         return i18n("invalid");
@@ -842,23 +928,23 @@ QIcon Formatting::validityIcon(const UserID::Signature &sig)
             case 0x11: /* Persona */
             case 0x12: /* Casual */
             case 0x13: /* Positive */
-                return QIcon::fromTheme(QStringLiteral("emblem-success"));
+                return Formatting::successIcon();
             case 0x30:
-                return QIcon::fromTheme(QStringLiteral("emblem-error"));
+                return Formatting::errorIcon();
             default:
                 return QIcon();
             }
         }
-        Q_FALLTHROUGH();
+        [[fallthrough]];
         // fall through:
     case UserID::Signature::BadSignature:
     case UserID::Signature::GeneralError:
-        return QIcon::fromTheme(QStringLiteral("emblem-error"));
+        return Formatting::errorIcon();
     case UserID::Signature::SigExpired:
     case UserID::Signature::KeyExpired:
-        return QIcon::fromTheme(QStringLiteral("emblem-information"));
+        return Formatting::infoIcon();
     case UserID::Signature::NoPublicKey:
-        return QIcon::fromTheme(QStringLiteral("emblem-question"));
+        return Formatting::questionIcon();
     }
     return QIcon();
 }
@@ -881,10 +967,7 @@ QString Formatting::formatForComboBox(const GpgME::Key &key)
     return i18nc("name, email, key id", "%1 %2 (%3)", name, mail, QLatin1String(key.shortKeyID())).simplified();
 }
 
-namespace
-{
-
-static QString keyToString(const Key &key)
+QString Formatting::nameAndEmailForSummaryLine(const Key &key)
 {
     Q_ASSERT(!key.isNull());
 
@@ -900,8 +983,6 @@ static QString keyToString(const Key &key)
     }
 }
 
-}
-
 const char *Formatting::summaryToString(const Signature::Summary summary)
 {
     if (summary & Signature::Red) {
@@ -925,12 +1006,12 @@ QString Formatting::signatureToString(const Signature &sig, const Key &key)
     if (red) {
         if (key.isNull()) {
             if (const char *fpr = sig.fingerprint()) {
-                return i18n("Bad signature by unknown certificate %1: %2", QString::fromLatin1(fpr), QString::fromLocal8Bit(sig.status().asString()));
+                return i18n("Bad signature by unknown certificate %1: %2", QString::fromLatin1(fpr), Formatting::errorAsString(sig.status()));
             } else {
-                return i18n("Bad signature by an unknown certificate: %1", QString::fromLocal8Bit(sig.status().asString()));
+                return i18n("Bad signature by an unknown certificate: %1", Formatting::errorAsString(sig.status()));
             }
         } else {
-            return i18n("Bad signature by %1: %2", keyToString(key), QString::fromLocal8Bit(sig.status().asString()));
+            return i18n("Bad signature by %1: %2", nameAndEmailForSummaryLine(key), Formatting::errorAsString(sig.status()));
         }
 
     } else if (valid) {
@@ -941,17 +1022,17 @@ QString Formatting::signatureToString(const Signature &sig, const Key &key)
                 return i18n("Good signature by an unknown certificate.");
             }
         } else {
-            return i18n("Good signature by %1.", keyToString(key));
+            return i18n("Good signature by %1.", nameAndEmailForSummaryLine(key));
         }
 
     } else if (key.isNull()) {
         if (const char *fpr = sig.fingerprint()) {
-            return i18n("Invalid signature by unknown certificate %1: %2", QString::fromLatin1(fpr), QString::fromLocal8Bit(sig.status().asString()));
+            return i18n("Invalid signature by unknown certificate %1: %2", QString::fromLatin1(fpr), Formatting::errorAsString(sig.status()));
         } else {
-            return i18n("Invalid signature by an unknown certificate: %1", QString::fromLocal8Bit(sig.status().asString()));
+            return i18n("Invalid signature by an unknown certificate: %1", Formatting::errorAsString(sig.status()));
         }
     } else {
-        return i18n("Invalid signature by %1: %2", keyToString(key), QString::fromLocal8Bit(sig.status().asString()));
+        return i18n("Invalid signature by %1: %2", nameAndEmailForSummaryLine(key), Formatting::errorAsString(sig.status()));
     }
 }
 
@@ -979,7 +1060,7 @@ QString Formatting::importMetaData(const Import &import)
         return i18n("The import of this certificate was canceled.");
     }
     if (import.error()) {
-        return i18n("An error occurred importing this certificate: %1", QString::fromLocal8Bit(import.error().asString()));
+        return i18n("An error occurred importing this certificate: %1", Formatting::errorAsString(import.error()));
     }
 
     const unsigned int status = import.status();
@@ -1031,7 +1112,7 @@ QString Formatting::usageString(const Subkey &sub)
 
 QString Formatting::summaryLine(const Key &key)
 {
-    return keyToString(key) + QLatin1Char(' ')
+    return nameAndEmailForSummaryLine(key) + QLatin1Char(' ')
         + i18nc("(validity, protocol, creation date)",
                 "(%1, %2, created: %3)",
                 Formatting::complianceStringShort(key),
@@ -1067,29 +1148,13 @@ QString Formatting::summaryLine(const KeyGroup &group)
     }
 }
 
-namespace
-{
-QIcon iconForValidity(UserID::Validity validity)
-{
-    switch (validity) {
-    case UserID::Ultimate:
-    case UserID::Full:
-    case UserID::Marginal:
-        return QIcon::fromTheme(QStringLiteral("emblem-success"));
-    case UserID::Never:
-        return QIcon::fromTheme(QStringLiteral("emblem-error"));
-    case UserID::Undefined:
-    case UserID::Unknown:
-    default:
-        return QIcon::fromTheme(QStringLiteral("emblem-information"));
-    }
-}
-}
-
 // Icon for certificate selection indication
 QIcon Formatting::iconForUid(const UserID &uid)
 {
-    return iconForValidity(uid.validity());
+    if (Kleo::isRevokedOrExpired(uid)) {
+        return Formatting::errorIcon();
+    }
+    return iconForValidity(uid);
 }
 
 QString Formatting::validity(const UserID &uid)
@@ -1134,16 +1199,31 @@ UserID::Validity minimalValidity(const Container &keys)
     });
     return minValidity <= UserID::Ultimate ? static_cast<UserID::Validity>(minValidity) : UserID::Unknown;
 }
+
+template<typename Container>
+bool allKeysAreCompliant(const Container &keys)
+{
+    if (!DeVSCompliance::isActive()) {
+        return true;
+    }
+    if (!DeVSCompliance::isCompliant()) {
+        return false;
+    }
+    return Kleo::all_of(keys, DeVSCompliance::keyIsCompliant);
+}
 }
 
 QIcon Formatting::validityIcon(const KeyGroup &group)
 {
-    return iconForValidity(minimalValidity(group.keys()));
+    if (Kleo::any_of(group.keys(), std::mem_fn(&Key::isBad))) {
+        return Formatting::errorIcon();
+    }
+    return iconForValidityAndCompliance(minimalValidity(group.keys()), allKeysAreCompliant(group.keys()));
 }
 
 bool Formatting::uidsHaveFullValidity(const Key &key)
 {
-    return minimalValidityOfNotRevokedUserIDs(key) >= UserID::Full;
+    return allUserIDsHaveFullValidity(key);
 }
 
 QString Formatting::complianceMode()
@@ -1154,16 +1234,7 @@ QString Formatting::complianceMode()
 
 bool Formatting::isKeyDeVs(const GpgME::Key &key)
 {
-    for (const auto &sub : key.subkeys()) {
-        if (sub.isExpired() || sub.isRevoked()) {
-            // Ignore old subkeys
-            continue;
-        }
-        if (!sub.isDeVs()) {
-            return false;
-        }
-    }
-    return true;
+    return DeVSCompliance::allSubkeysAreCompliant(key);
 }
 
 QString Formatting::complianceStringForKey(const GpgME::Key &key)
@@ -1173,18 +1244,18 @@ QString Formatting::complianceStringForKey(const GpgME::Key &key)
     if (DeVSCompliance::isCompliant()) {
         return isRemoteKey(key) //
             ? i18nc("@info the compliance of the key with certain requirements is unknown", "unknown")
-            : DeVSCompliance::name(uidsHaveFullValidity(key) && isKeyDeVs(key));
+            : DeVSCompliance::name(DeVSCompliance::keyIsCompliant(key));
     }
     return QString();
 }
 
 QString Formatting::complianceStringShort(const GpgME::Key &key)
 {
+    if (DeVSCompliance::isCompliant() && DeVSCompliance::keyIsCompliant(key)) {
+        return QStringLiteral("★ ") + DeVSCompliance::name(true);
+    }
     const bool keyValidityChecked = (key.keyListMode() & GpgME::Validate);
-    if (keyValidityChecked && Formatting::uidsHaveFullValidity(key)) {
-        if (DeVSCompliance::isCompliant() && Formatting::isKeyDeVs(key)) {
-            return QStringLiteral("★ ") + DeVSCompliance::name(true);
-        }
+    if (keyValidityChecked && Kleo::allUserIDsHaveFullValidity(key)) {
         return i18nc("As in all user IDs are valid.", "certified");
     }
     if (key.isExpired()) {
@@ -1210,7 +1281,7 @@ QString Formatting::complianceStringShort(const KeyGroup &group)
 {
     const KeyGroup::Keys &keys = group.keys();
 
-    const bool allKeysFullyValid = std::all_of(keys.cbegin(), keys.cend(), &Formatting::uidsHaveFullValidity);
+    const bool allKeysFullyValid = std::all_of(keys.cbegin(), keys.cend(), &Kleo::allUserIDsHaveFullValidity);
     if (allKeysFullyValid) {
         return i18nc("As in all keys are valid.", "all certified");
     }
@@ -1308,3 +1379,13 @@ QString Formatting::trustSignature(const GpgME::UserID::Signature &sig)
         return {};
     }
 }
+
+QString Formatting::errorAsString(const GpgME::Error &error)
+{
+#ifdef Q_OS_WIN
+    // On Windows, we set GpgME resp. libgpg-error to return (translated) error messages as UTF-8
+    return QString::fromUtf8(error.asString());
+#else
+    return QString::fromLocal8Bit(error.asString());
+#endif
+}
