@@ -25,6 +25,7 @@ namespace Kleo
 {
 
 KLEO_EXPORT QString gnupgHomeDirectory();
+KLEO_EXPORT QString gnupgPrivateKeysDirectory();
 
 KLEO_EXPORT QString gpgConfPath();
 KLEO_EXPORT QString gpgSmPath();
@@ -93,8 +94,6 @@ KLEO_DEPRECATED_EXPORT bool gnupgUsesDeVsCompliance();
  */
 KLEO_DEPRECATED_EXPORT bool gnupgIsDeVsCompliant();
 
-KLEO_EXPORT enum GpgME::UserID::Validity keyValidity(const GpgME::Key &key);
-
 /* Convert GnuPG output to a QString with proper encoding.
  * Takes Gpg Quirks into account and might handle future
  * changes in GnuPG Output. */
@@ -117,4 +116,18 @@ KLEO_EXPORT void launchGpgAgent();
  */
 KLEO_EXPORT void killDaemons();
 
+/**
+ * Returns a static list of the available algorithms.
+ */
+KLEO_EXPORT const std::vector<std::string> &availableAlgorithms();
+
+/**
+ * Returns a static list of the preferred algorithms with decreasing preference.
+ */
+KLEO_EXPORT const std::vector<std::string> &preferredAlgorithms();
+
+/**
+ * Returns a static list of algorithms that are explicitly not supported.
+ */
+KLEO_EXPORT const std::vector<std::string> &ignoredAlgorithms();
 }
